#include <jabber/jabber.h>
#include <etherx.h>

extern pth_mutex_t s__m;
extern xmlnode irc_agents;
extern xmlnode irc_trans_config;
extern ehandler ehandler_irc;
extern ehandler ehandler_reg;
extern char *irc_host;
extern char *reg_host;
extern char *etherx_host;
extern char *etherx_secret;
extern char *default_server;
extern char *default_port;
extern int irc_register;
extern pool main_pool;

typedef struct user_list {
    char *user;
    int ops;
    int voice;
    struct user_list *next;
} _users, *users;


extern users part_list;
extern users quit_list;

typedef struct chan_struct {
    pool p;
    char *channel;
    char *id;
    users user_list;
    struct chan_struct *next;
} _chans, *chans;

typedef struct sessions_struct
{
    pool p;
    thread t;
    thread stream;
    jid j;
    int rc; /* ref count */
    int default_session;
    users notify_list;
    int port;
    char *nick;
    char *server;
    char *pass;
    chans channel_list;
    char *buffer;
    struct sessions_struct *prev;
    struct sessions_struct *next;
} _session, *session;

extern session sessions;
thread irc_session_create(jpacket jp,int default_conn,char *pass);
void irc_session_end(session s,char *channel,char *reason);
int irc_session_isdone( void *arg );
session irc_session_find_by_jid(jid j,char *server);
void session_unlink(session s);

/* util.c */
void irc_update_notify_pres(session s);
void svc_irc_line(session s, char *cmd, char *data, int argc, ...);
int j_cmp_nores(jid a,jid b);
char *irc_get_id(session s,char *channel);
int irc_nick_cmp(char *nick1,char *nick2);
void yt_strip_jid(jid j);
void irc_del_user(session s,char *chan, char *nick);
void irc_set_mode(users u,int ops, int voice);
users irc_add_user(session s,char *chan, char *nick);
char *irc_get_port(pool p,char *input);
users irc_get_user(session s, char *channel,char *nick);
int irc_channel_cmp(const char *chan1,const char *chan2);
void irc_send_error(jid to,char *from,char *code,char *msg);
char *irc_get_server(pool p,char *input);
char *irc_get_channel(pool p,char *input);
void irc_send_message(session s,char *channel,char *nick,char *type,char *body, char *subject);
void irc_send_presence(session s,char *channel,char *nick,int type,char *status);
void irc_del_user_by_id(session s,char *id,char *nick);
users irc_add_user_by_id(session s,char *id,char *nick);
char *irc_get_id_by_user(session s,char *nick);
void irc_strip_mirc_codes(char *input);
chans irc_find_channel_by_id(session s,char *id);
chans irc_find_channel_by_name(session s,char *channel);
void _debug_print_sessions(void);
void irc_add_notify(session s,char *nick);
void irc_del_notify(session s,char *nick);
char *irc_get_part_msg(void);
char *irc_get_quit_msg(void);

/* external.c */
void irc_init();
char *str_to_UTF8(pool p,unsigned char *in);
void irc_etherx_receive(epacket e);
void irc_etherx_reg_receive(epacket e);
unsigned char *UTF8_to_str(pool p,unsigned char *in);


/* config.c */
int irc_handle_config(xmlnode conf);
int load_config(char* configfile);

/* control.c */
int irc_trans_stop(void);

/* parser.c */
int irc_parse_packet(jpacket jp);

/* register.c */
int irc_parse_reg_available(jpacket jp);
int irc_parse_reg_packet(jpacket jp);
session irc_find_default_session(jid j);

/* from_irc.c */
void irc_handle_ison(session s,pool p,char *from,char **argv);
int parse_server(pool p,session s,char *input);

