/*
IRC-Transport config.c Reads the config XML file
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/
#include <irctrans.h>

xmlnode irc_trans_config = NULL;
xmlnode irc_agents=NULL;
char *etherx_host=NULL;
char *etherx_secret=NULL;
char *irc_host=NULL;
char *reg_host=NULL;
char *default_server=NULL;
char *default_port=NULL;
int irc_register=0;
users part_list=NULL,quit_list=NULL;


/*
does the actual parsing of the config file
*/
int handle_config(xmlnode conf)
/*
returns:
  0 upon success
  1 upon error
input:
  xmlnode conf: the xmlnode representation of the local config file
*/
{
    xmlnode next;
    /* we got the chunk from the config file, 
    make sure we init everything depending on config reguardless if they were configured  */
    next = xmlnode_get_firstchild(conf);
    while(next != NULL)
    {
        /* 
        <etherx/> section may contain <host/> and <secret/> tags,
        used to connect with etherx
        */
        if(j_strcmp(xmlnode_get_name(next),"agents")==0)
        {
            irc_agents=next;
            reg_host=pstrdup(main_pool,xmlnode_get_attrib(xmlnode_get_tag(irc_agents,"agent"),"jid"));
        }
        else if(j_strcmp(xmlnode_get_name(next),"etherx")==0)
        {
            xmlnode etherx = xmlnode_get_firstchild(next);
            while( etherx != NULL )
            {
                if(j_strcmp(xmlnode_get_name(etherx),"host")==0)
                {
                    log_debug(ZONE,"[init] Using etherx host: %s", xmlnode_get_data(etherx));
                    etherx_host = pstrdup(main_pool, xmlnode_get_data(etherx));
                }
                else if(j_strcmp(xmlnode_get_name(etherx),"secret")==0)
                {
                    log_debug(ZONE,"[init] Using etherx secret: %s", xmlnode_get_data(etherx));
                    etherx_secret = pstrdup(main_pool,xmlnode_get_data(etherx));
                 }
                 etherx = xmlnode_get_nextsibling(etherx);
             }
        }
        /* 
        <name/> tag contains the name of the irc transport
        */
        else if(j_strcmp(xmlnode_get_name(next),"name") == 0)
        {
	    log_debug(ZONE, "[init] using alias: %s\n", xmlnode_get_data(next));
            irc_host = pstrdup(main_pool,xmlnode_get_data(next));
	}
        /*
        a <register/> flag indicates that the transport allows registration
        */
        else if(j_strcmp(xmlnode_get_name(next),"register")==0)
            irc_register=1;
        /*
        <default/> tag specifies the default <server/> and <port/>, if none are
        specified, then irc.linux.com:6667 is used
        */
        else if(j_strcmp(xmlnode_get_name(next),"default")==0)
        {
            xmlnode server=xmlnode_get_firstchild(next);
            while(server!=NULL)
            {
                if(j_strcmp(xmlnode_get_name(server),"server")==0)
                {
                    log_debug(ZONE,"[init] using default server: %s",xmlnode_get_data(server));
                    default_server=pstrdup(main_pool,xmlnode_get_data(server));
                }
                else if(j_strcmp(xmlnode_get_name(server),"port")==0)
                {
                    log_debug(ZONE,"init] using default port: %s",xmlnode_get_data(server));
                    default_port=pstrdup(main_pool,xmlnode_get_data(server));
                }
                server=xmlnode_get_nextsibling(server);
            }
        }
        else if(j_strcmp(xmlnode_get_name(next),"quit")==0)
        {
            users first,curr;
            xmlnode list;
            first=quit_list;   
            list=xmlnode_get_firstchild(next);
            while(list!=NULL&&j_strcmp(xmlnode_get_name(list),"message")!=0)
                list=xmlnode_get_nextsibling(list);    
            if(list!=NULL)
            {
                quit_list=pmalloc(main_pool,sizeof(_users));
                first=quit_list;
                first->user=pstrdup(main_pool,xmlnode_get_data(xmlnode_get_firstchild(list)));
                first->next=first; 
                while((list=xmlnode_get_nextsibling(list))!=NULL)
                {
                    if(j_strcmp(xmlnode_get_name(list),"message")==0)    
                    {
                        curr=pmalloc(main_pool,sizeof(_users));   
                        curr->user=pstrdup(main_pool,xmlnode_get_data(xmlnode_get_firstchild(list)));
                        quit_list->next=curr;
                        quit_list=quit_list->next;
                        quit_list->next=first;
                    }
                }
            }
        }
        else if(j_strcmp(xmlnode_get_name(next),"part")==0)
        {
            users first,curr;
            xmlnode list;
            first=quit_list;   
            list=xmlnode_get_firstchild(next);
            while(list!=NULL&&j_strcmp(xmlnode_get_name(list),"message")!=0)
                list=xmlnode_get_nextsibling(list);    
            if(list!=NULL)
            {
                part_list=pmalloc(main_pool,sizeof(_users));
                first=part_list;
                first->user=pstrdup(main_pool,xmlnode_get_data(xmlnode_get_firstchild(list)));
                first->next=first; 
                while((list=xmlnode_get_nextsibling(list))!=NULL)
                {
                    if(j_strcmp(xmlnode_get_name(list),"message")==0)    
                    {
                        curr=pmalloc(main_pool,sizeof(_users));   
                        curr->user=pstrdup(main_pool,xmlnode_get_data(xmlnode_get_firstchild(list)));
                        part_list->next=curr;
                        part_list=part_list->next;
                        part_list->next=first;
                    }
                }
            }
        }
        next = xmlnode_get_nextsibling(next);
    }
    if( etherx_host == NULL ) 
    { /* no etherx host was found, use 127.0.0.1 instead */
        log_error(ZONE,"[CONFIG] Unable to find Etherx host in config, defaulting to 127.0.0.1");
        etherx_host=pstrdup(main_pool,"127.0.0.1");
    }
    if( etherx_secret == NULL ) 
    { /* no etherx secret was found, use 'test' */
        log_error(ZONE,"[CONFIG] Unable to find Etherx secret in config, defaulting to 'test'");
        etherx_secret=pstrdup(main_pool,"test");
    }
    if( irc_host == NULL ) 
    { /* no host name specified, no good way to handle this, exit */
        log_error(ZONE,"[CONFIG] Unable to find IRC host name in config");
        return 1;
    }
    if(reg_host==NULL)
    {
        spool sp=spool_new(main_pool);
        spooler(sp,"register.",irc_host,sp);
        reg_host=spool_print(sp);
    }
    /* default the server to irc.linux.com:6667 if not otherwise specified */
    if(default_port==NULL)
        default_port=pstrdup(main_pool,"6667");
    if(default_server==NULL)
        default_server=pstrdup(main_pool,"irc.linux.com");
    return 0;
}

/*
loads a config file into global variables
*/
int load_config(char *configfile) 
/*
returns: 
  0 upon success
  1 upon failure
input:
  char *configfile: the name of the local file that houses the config
*/
{
    /* if no config is specified, use the default location */
    if(configfile==NULL) configfile=IRCTRANSCONF;
    irc_trans_config=xmlnode_file(configfile);
    if(irc_trans_config==NULL) return 1;
    log_warn(ZONE,"[init] reading config from (%s)", configfile);
    /* perform the actual parsing */
    return handle_config(irc_trans_config);
}
