/*
IRC-Transport external.c: Handles connections to etherx
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <irctrans.h>

ehandler ehandler_irc;
ehandler ehandler_reg;

/*
initializes etherx connections
*/
void irc_init(void)
/*
returns:
  nothing
input:
  none
*/
{
    ehandler_reg=NULL;
    /* register both irc.foo.dom and register.irc.foo.dom */
    ehandler_irc = ehandler_new(&irc_etherx_receive, irc_host,"jabber:server");
    if(irc_register)
    { /* if there is a <register/> flag in the config */
        ehandler_reg=ehandler_new(&irc_etherx_reg_receive,reg_host,"jabber:server");
    }
    if(ehandler_irc==NULL)
    { 
        log_error(ZONE, "[init] Failed to connect with etherx");
        pool_free(main_pool);
        exit(1);
    }
    return;
}

/* 
recieves a packet on register.irc.foo.dom
*/
void irc_etherx_reg_receive(epacket e)
/*
returns:
  nothing
input:
  epacket e: an epacket, sent from etherx to register.irc.foo.dom 
*/
{
    jpacket jp;
    if(e==NULL)return;
    log_debug(ZONE,"%s: %s",reg_host,xmlnode2str(e->x));
    switch(e->type)
    {
    case EPACKET_NORMAL:
        /* this is a normal packet, deliver it to the parser */
        xmlnode_hide_attrib(e->x,"etherx:to");
        xmlnode_hide_attrib(e->x,"etherx:from");
        jp=jpacket_new(e->x);
        irc_parse_reg_packet(jp);
        break;
    case EPACKET_ERROR:
        /* error packet or unknown, drop it */
        xmlnode_free(e->x);
        break;
    default:
        xmlnode_free(e->x);
        break;
    }
    _debug_print_sessions();
}            
            

/* 
recieves a packet on irc.foo.dom
*/
void irc_etherx_receive(epacket e)
/*
returns:
  nothing
input:
  epacket e: an epacket, sent from etherx to irc.foo.dom 
*/
{
    jpacket jp;
    if(e==NULL)return;
    log_debug(ZONE, "%s: %s", irc_host,xmlnode2str(e->x));
    switch(e->type)
    {
        case EPACKET_NORMAL:
            /* normal packet.. deliver to the parser */
            xmlnode_hide_attrib(e->x, "etherx:to");
            xmlnode_hide_attrib(e->x, "etherx:from");
            jp = jpacket_new( e->x );
            if(irc_parse_packet(jp))
                log_error(ZONE,"ERROR HANDLE OF PACKET");
            break;
    case EPACKET_ERROR:
            /* error packet, or unknown, drop it */
            xmlnode_free(e->x);
            break;
        default:
            xmlnode_free(e->x);
    }
    _debug_print_sessions();
}

/*
add a single char to a string
*/
char *addchar(pool p,char *in,char new)
/*
returns:
  the new string, with the char added
input:
  pool p:       a valid pool to use to add the char
  char *in:     a string to add the new char to
  char new:     a single character to add to the string
*/
{
    char *ret=pmalloc(p,2);
    spool sp=spool_new(p);
    sprintf(ret,"%c",new);
    if(in==NULL)
        spooler(sp,ret,sp);
    else
        spooler(sp,in,ret,sp);
    return spool_print(sp);
}

/*
converts a normal string to UTF8 format
*/
char *str_to_UTF8(pool p,unsigned char *in)
/*
returns:
  the new string, encoded in UTF-8
input:
  pool p:   a pool to define the life of the return value
  unsigned char *in: the str to convert
*/
{
    char *result=NULL;
    int n;
    if(in==NULL||p==NULL) return NULL;
    for(n=0;n<strlen(in);n++)
    {
        long c=(long)in[n];
        if(c<128)
        {
            result=addchar(p,result,(char)c);
        }
        else
        {
            result=addchar(p,result,(char)((c>>6)|192));
            result=addchar(p,result,(char)((c&63)|128));
        }
    }
    return result;
}

/*
convert a UTF-8 encoded string to a normal string
*/
unsigned char *UTF8_to_str(pool p,unsigned char *in)
/*
returns:
  a string not encoded in UTF-8
input:
  pool p: a pool to define the life of the return value
  unsigned char *in: the UTF-8 encoded string
*/
{
    int n;
    char *result=NULL;
    if(p==NULL||in==NULL) return NULL;
    n = 0;
    while(n<=strlen(in)-1)
    {
        long c=(long)in[n];
        if(c<0x80)
        {
            result=addchar(p,result,(char)c);
        }
        else
        {
            if((c&0xC0)==0xC0)
            {
                result=addchar(p,result,(char)(((c&0x03)<<6)|(((long)in[++n])&0x3F)));
            }
            else if((c&0xE0)==0xE0)
            {
                result=addchar(p,result,(char)(((c&0xF)<<4)|(((long)in[++n])&0x3F)|(((long)in[++n])&0x3F)));
            }
            else
                n++; /*skip that char*/
        }
        n++;
    }
        return result;
}
