/*
IRC-Transport from_irc.c: parses data from an IRC connection
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include "irctrans.h"

/*
ISON is used to poll the server for nick presence.
*/
void irc_handle_ison(session s,pool p,char *from,char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
*/
{
    char *next,*curr;
    users n=s->notify_list;
    if(s==NULL||p==NULL||argv==NULL)return;
    curr=argv[1];
    /* zero out the ->voice */
    for(n=s->notify_list;n!=NULL;n->voice=0,n=n->next);
    
    while(curr!=NULL)
    {
        next=strchr(curr,' ');
        if(next!=NULL)
        {
            next[0]='\0';
            next++;
        /* find this user and set it's ->voice */
        for(n=s->notify_list;n!=NULL;n=n->next)
            if(irc_nick_cmp(curr,n->user)==0)
                n->voice=1;
        }
        curr=next;
    }
    irc_update_notify_pres(s);
}

/*
display the IRC error and exit the session
*/
void irc_handle_error(session s,pool p,char *from,char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv[0]: the error message
*/
{
    if(s==NULL||p==NULL||argv==NULL) return;
    irc_send_message(s,NULL,NULL,"groupchat",argv[0],NULL);
    session_unlink(s);
    tstream_close(s->stream);
}

/*
sets the proper user mode (ops/voice only) and updates the username
*/
void irc_handle_mode(session s,pool p,char *from,char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv[0]: channel
    argv[1]: opts
    argv[2]: user
*/
{
   int val,argc=2;
   char *opts=argv[1],*nick=strtok(from,"!");
   if(s==NULL||p==NULL||from==NULL||argv[1]==NULL||argv[0]==NULL) return;
   if(opts[0]=='+') val=1;
   if(opts[0]=='-') val=0;
   opts++;
   /* loop through the opts ie.. +ooo would be three loops */
   while(opts[0]!='\0')
   {
       users u;
       spool sp=spool_new(p);
       u=irc_get_user(s,argv[0],argv[argc]);
       if(u==NULL) u=irc_add_user(s,argv[0],argv[argc]);
       if(u==NULL) return;
       irc_set_mode(u,0,0);
       if(opts[0]=='o')
       { /* ops */
           if(val) 
               spooler(sp,nick," Has Given ",argv[argc]," Operator Status",sp);
           else 
               spooler(sp,nick," Has taken Operator Status from ",argv[argc],sp);
           irc_send_message(s,argv[0],NULL,"groupchat",spool_print(sp),NULL);
       }
       else if(opts[0]=='v')
       { /* voice */
           if(val) 
               spooler(sp,nick," Has Given ",argv[argc]," Voice",sp);
           else 
               spooler(sp,nick," Has Removed Voice from ",argv[argc],sp);
           irc_send_message(s,argv[0],NULL,"groupchat",spool_print(sp),NULL);
       }
       else
       { /* not ops or voice, we don't care */
           opts++;
           continue;
       }
       if(irc_nick_cmp(s->nick,argv[argc])==0)
       {  /* if this change is for us... */
           spool nicksp=spool_new(p);
           xmlnode iq,query,nick;
           spool sp=spool_new(p);
           if(opts[0]=='o')
           {
               u->ops=val;
               if(val) spooler(nicksp,"@",argv[argc],nicksp);
               else spooler(nicksp,argv[argc],nicksp);
               spooler(sp,irc_get_id(s,argv[0]),"@",irc_host,sp);
           } 
           else if(opts[0]=='v')
           {
               u->voice=val;
               if(val) spooler(nicksp,"@",argv[argc],nicksp);
               else spooler(nicksp,argv[argc],nicksp);
               spooler(sp,irc_get_id(s,argv[0]),"+",irc_host,sp);
           } 
           /* send the special notifier that it's for us */
           /*
            NOTE: this is an ugly hack to get it working quick
            winjab and gabber now support this, and it needs to
            be done better, IMHO before it becomes 'mainstream'
           */
           iq=xmlnode_new_tag("iq");
           xmlnode_put_attrib(iq,"to",jid_full(s->j));
           xmlnode_put_attrib(iq,"from",spool_print(sp));
           xmlnode_put_attrib(iq,"type","set");
           query=xmlnode_insert_tag(iq,"query");
           xmlnode_put_attrib(query,"xmlns","jabber:x:gc");
           nick=xmlnode_insert_tag(query,"nick");
           xmlnode_insert_cdata(nick,spool_print(nicksp),-1);
           ehandler_send(ehandler_irc,iq,s->j->server);
       }
       else
       { /* change the users' nick */
           irc_send_presence(s,argv[0],argv[argc],JPACKET__UNAVAILABLE,NULL);
           if(opts[0]=='o')
               u->ops=val;
           else if(opts[0]=='v')
               u->voice=val;
           irc_send_presence(s,argv[0],argv[argc],JPACKET__AVAILABLE,NULL);
       }
       opts++;
       argc++;
    }
}

/* 
when a user joins, display a message, and send presence
*/
void irc_handle_join(session s,pool p,char *from,char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv[0]: channel
*/
{
    spool sp=spool_new(p);
    char *nick=strtok(from,"!");
    char *channel=argv[0];
    spooler(sp,nick," Has Joined Channel ",channel,sp);
    irc_send_message(s,channel,NULL,"groupchat",spool_print(sp),NULL);
    irc_set_mode(irc_add_user(s,channel,nick),0,0); 
    irc_send_presence(s,channel,nick,JPACKET__AVAILABLE,NULL);
}

/*
send a part message and unavail presence
*/
void irc_handle_part(session s,pool p,char *from,char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv[0]: channel
    argv[1]: reason to part
*/
{
    char *nick=strtok(from,"!");
    char *channel=argv[0];
    spool sp=spool_new(p);
    spooler(sp,nick," Has Left Channel ",channel,sp);
    irc_send_message(s,channel,NULL,"groupchat",spool_print(sp),NULL);
    irc_send_presence(s,channel,nick,JPACKET__UNAVAILABLE,NULL);
    irc_del_user(s,channel,nick);
}

/*
show a quit message and send unavail
*/
void irc_handle_quit(session s,pool p,char *from,char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv[0]: quit reason
*/
{
    spool sp=spool_new(p);
    char *nick=strtok(from,"!");
    char *id=irc_get_id_by_user(s,nick);
    while(id!=NULL)
    {
        chans curr=irc_find_channel_by_id(s,id);
        sp=spool_new(p);
        spooler(sp,nick," Has quit IRC: ",argv[0],sp);
        irc_send_message(s,curr->channel,NULL,"groupchat",spool_print(sp),NULL);
        irc_send_presence(s,curr->channel,nick,JPACKET__UNAVAILABLE,NULL);
        irc_del_user(s,curr->channel,nick);
        id=irc_get_id_by_user(s,nick);
    }
}

/*
show a kick message and send unavail.. if this is for us, auto-rejoin
*/
void irc_handle_kick(session s,pool p,char *from,char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv[0]: channel
    argv[1]: nick
    argv[2]: reason
*/
{
    spool sp=spool_new(p);
    char *nick=strtok(from,"!");
    spooler(sp,argv[1]," was kicked by ",nick," (",argv[2],")",sp);
    irc_send_message(s,argv[0],NULL,"groupchat",spool_print(sp),NULL);
    irc_send_presence(s,argv[0],argv[1],JPACKET__UNAVAILABLE,NULL);
    if(j_strcmp(argv[0],s->nick)==0)
    {
        /* we were kicked from the channel */
        sp=spool_new(p);
        spooler(sp,"JOIN ",argv[0],"\n",sp);
        tstream_write(s->stream,spool_print(sp),-1);
    }
}

/*
change the user's internal nick, change the nick (presence) and send a message
*/
void irc_handle_nick_change(session s,pool p,char *from, char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv[0]: new nick
*/
{
    char *old_nick=strtok(from,"!");
    char *new_nick=argv[0];
    char *id=irc_get_id_by_user(s,old_nick);
    spool sp=spool_new(p);
    spooler(sp,old_nick," is now known as ",new_nick,sp);
    while(id!=NULL)
    { /* while we still find this user in the channel list */
        chans curr=irc_find_channel_by_id(s,id);
        if(curr==NULL) return; /* no good way to handle this */
        irc_send_message(s,curr->channel,NULL,"groupchat",spool_print(sp),NULL);
        if(irc_nick_cmp(old_nick,s->nick)==0)
        { /* this is our nick send a special message */
            users old_user=irc_get_user(s,curr->channel,old_nick);
            xmlnode iq,query,nick;
            spool sp=spool_new(p);
            spooler(sp,curr->id,"@",irc_host,sp);
            iq=xmlnode_new_tag("iq");
            xmlnode_put_attrib(iq,"to",jid_full(s->j));
            xmlnode_put_attrib(iq,"from",spool_print(sp));
            xmlnode_put_attrib(iq,"type","set");
            query=xmlnode_insert_tag(iq,"query");
            xmlnode_put_attrib(query,"xmlns","jabber:x:gc");
            nick=xmlnode_insert_tag(query,"nick");
            xmlnode_insert_cdata(nick,new_nick,-1);
            ehandler_send(ehandler_irc,iq,s->j->server);
            if(old_user!=NULL)
                irc_set_mode(irc_add_user(s,curr->channel,new_nick),old_user->ops,old_user->voice);
            else
                irc_set_mode(irc_add_user(s,curr->channel,new_nick),0,0);
            irc_del_user(s,curr->channel,old_nick);
        }
        else
        { /* nick change for someone else, update their nick */
            users old_user=irc_get_user(s,curr->channel,old_nick);
            irc_send_presence(s,curr->channel,old_nick,JPACKET__UNAVAILABLE,"Changing Nick");
            if(old_user!=NULL)
                irc_set_mode(irc_add_user(s,curr->channel,new_nick),old_user->ops,old_user->voice);
            else
                irc_set_mode(irc_add_user(s,curr->channel,new_nick),0,0);
            irc_del_user(s,curr->channel,old_nick);
            irc_send_presence(s,curr->channel,new_nick,JPACKET__AVAILABLE,NULL);
        }
        id=irc_get_id_by_user(s,old_nick);
    }
    if(j_strcmp(old_nick,s->nick)==0)
    {
        s->nick=pstrdup(s->t->p,new_nick);
    }
}

/*
roster sent when a user first joins the channel
*/
void irc_handle_roster(session s,pool p,char *from,char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv0 = your nick
    argv1 = '='
    argv2 = the channel
    argv3 = "<user> <user> <user>" with or without a trailing space
*/
{
     char *curr=argv[3];
     char *next=curr;
     char *channel=argv[2];
     users curr_user;
     if(channel==NULL||curr==NULL) return;
     while(1)
     { /* loop through the users */
       spool sp=spool_new(p);
       int exit=0;
       next=strchr(curr,' ');
       if(next==NULL)
       {
         exit=1;
       }
       else
       {
          next[0]='\0';
          next++;
          if(next[0]=='\0') exit=1;
       }
       /* in addition, add the user list to the channel list.. */
       curr_user=irc_add_user(s,argv[2],curr);
       irc_set_mode(curr_user,0,0);
       /* send presence */
       if(curr_user==NULL)
          break; /* something is pretty wrong here */
       spooler(sp,curr_user->user," Is in Channel ",argv[2],sp);
       irc_send_message(s,argv[2],NULL,"groupchat",spool_print(sp),NULL);
       irc_send_presence(s,argv[2],curr_user->user,JPACKET__AVAILABLE,NULL);
       if(curr[0]=='@')
       {
         sp=spool_new(p);
         spooler(sp,curr_user->user," Has Channel Operator Status",sp);
         irc_send_message(s,argv[2],NULL,"groupchat",spool_print(sp),NULL);
         if(irc_nick_cmp(curr,s->nick)==0)
         {
            spool nicksp=spool_new(p);
            xmlnode iq,query,nick;
            spool sp=spool_new(p);
            curr_user->ops=1;
            spooler(nicksp,"@",curr_user->user,nicksp);
            spooler(sp,irc_get_id(s,argv[2]),"@",irc_host,sp);
            iq=xmlnode_new_tag("iq");
            xmlnode_put_attrib(iq,"to",jid_full(s->j));
            xmlnode_put_attrib(iq,"from",spool_print(sp));
            xmlnode_put_attrib(iq,"type","set");
            query=xmlnode_insert_tag(iq,"query");
            xmlnode_put_attrib(query,"xmlns","jabber:x:gc");
            nick=xmlnode_insert_tag(query,"nick");
            xmlnode_insert_cdata(nick,spool_print(nicksp),-1);
            ehandler_send(ehandler_irc,iq,s->j->server);
         }
         else
         {
            irc_send_presence(s,argv[2],curr_user->user,JPACKET__UNAVAILABLE,"Given Ops");
            curr_user->ops=1;
            irc_send_presence(s,argv[2],curr_user->user,JPACKET__AVAILABLE,"Given Ops");
         }
       }
       else if(curr[0]=='+')
       {
         sp=spool_new(p);
         spooler(sp,curr_user->user," Has Voice",sp);
         irc_send_message(s,argv[2],NULL,"groupchat",spool_print(sp),NULL);
         irc_send_presence(s,argv[2],curr,JPACKET__UNAVAILABLE,NULL);
         curr_user->voice=1;
         irc_send_presence(s,argv[2],curr,JPACKET__AVAILABLE,NULL);
       }
       if(exit) break;
       curr=next;
     }
}

/*
handles server responses with 2 arguments in them..
in the form "arg0: arg1"
*/
void irc_handle_error2arg(session s,pool p,char *from, char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
*/
{
     spool sp=spool_new(p);
     spooler(sp,argv[0],": ",argv[1],sp);
     irc_send_message(s,NULL,NULL,"groupchat",spool_print(sp),NULL);
}

/*
handles server responses with 3 arguments in them..
in the form "arg0: arg1: arg2"
*/
void irc_handle_error3arg(session s,pool p,char *from, char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
*/
{
     spool sp=spool_new(p);
     spooler(sp,argv[0],": ",argv[1],": ",argv[2],sp);
     irc_send_message(s,NULL,NULL,"groupchat",spool_print(sp),NULL);
}

/*
handles a NOTICE from IRC... 
from arg may be null if this is a server NOTICE
*/
void irc_handle_notice(session s,pool p,char *from, char **argv)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv[0] -- channel/user
    argv[1] -- message
*/
{
     spool sp=spool_new(p);
     if(from!=NULL)
     {
        char *nick=strtok(from,"!");
        spooler(sp,nick,": ",argv[1],sp);
     }
     else
       spooler(sp,argv[1],sp);
     irc_send_message(s,NULL,NULL,"groupchat",spool_print(sp),NULL);
}

/*
handles a PRIVMSG from IRC... 
this function will notify the jabber use of the message, either as a "chat" or "groupchat"
where appropriate.
*/
void irc_handle_privmsg(session s,pool p, char *from, char **argv )
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  char *from:  the sender of this command
  char **argv: the arguments
    argv[0] -- channel/user
    argv[1] -- message
*/
{
    char *msg=argv[1];
    spool sp=spool_new(p);
    char *nick,*channel=argv[0];
    nick=strtok(from,"!");
    if(argv[1]==NULL) return;
    if(strncmp(argv[1],"\001ACTION",7)==0)
    {
        msg+=4;
        msg[0]='/';
        msg[1]='m';
        msg[2]='e';
        msg[strlen(msg)-1]='\0';
    }
    else if(strncmp(argv[1],"\001",1)==0)
    { /* a CTCP request */
        if(strncmp(argv[1],"\001PING",5)==0)
        {
            spooler(sp,"\r\nNOTICE ", from," :",argv[1],"\r\n",sp);
            tstream_write(s->stream,spool_print(sp),-1);
        }
        if(strncmp(argv[1],"\001VERSION",8)==0)
        {
            spooler(sp,"\r\nNOTICE ",from," :\001VERSION Jabber (www.jabber.org) IRC-Transport -- Use it, love it.\001\r\n",sp);
            tstream_write(s->stream,spool_print(sp),-1);
        }
        if(strncmp(argv[1],"\001FINGER",7)==0)
        {
            spooler(sp,"\r\nNOTICE ",from," :\001FINGER Jabber (www.jabber.org) user, JID: ",jid_full(s->j),"\r\n",sp);
            tstream_write(s->stream,spool_print(sp),-1);
        }
        if(strncmp(argv[1],"\001TIME",5)==0)
        {
            spooler(sp,"\r\nNOTICE ",from," :\001TIME ",jutil_timestamp(),"\001\r\n",sp);
            tstream_write(s->stream,spool_print(sp),-1);
        }
        return;
    }
    channel=argv[0];
    if(irc_get_id(s,channel)!=NULL)
    {
        /* normal message from the channel */
        irc_send_message(s,channel,nick,"groupchat",msg,NULL);
    }
    else
    {
        xmlnode message=xmlnode_new_tag("message");
        sp=spool_new(xmlnode_pool(message));
        if(s->default_session)
        {
            /* find if the nick is in our notify list */
            users next=s->notify_list;
            while(next!=NULL)
            {
                if(irc_nick_cmp(next->user,nick)==0)
                    spooler(sp,nick,"@",reg_host,sp);
                next=next->next;
            }
        }
        else
            spooler(sp,irc_get_id_by_user(s,channel),"@",irc_host,"/",nick,sp);
        xmlnode_put_attrib(message,"from",spool_print(sp));
        xmlnode_put_attrib(message,"to",jid_full(s->j));
        xmlnode_put_attrib(message,"type","chat");
        xmlnode_insert_cdata(xmlnode_insert_tag(message,"body"),msg,-1);
        ehandler_send(ehandler_irc,message,s->j->server);    
    }            
}

/*
takes data from IRC, parses it and calls the appropriate function
*/
int parse_server (pool p,session s,char *input)
/*
returns:
  nothing
input:
  session s:   this users session
  pool p:      temp storage location
  session s:   this users session
  char *input  the data to parse
*/
{
  char **lines=pmalloc(p,sizeof(char*)*100);
  char *last,*next;
  int linec=0;
  char *from=NULL;
  char *command=NULL;
  char **argv;
  int argc=0;
  int foo=0;
  static int LoopCount=0;

  if(input==NULL||s==NULL||s->rc==0||p==NULL) return 1;
  LoopCount++;
  if(LoopCount>=90)
  { /* we made 90 passes on this input.. that's over 90 lines in one read */
    LoopCount=0;
    return 0;
  }
  pth_mutex_acquire(&s__m,0,NULL);
  for(linec=0;linec<=95;linec++) lines[linec]=NULL;
  linec=0;
  lines[linec]=strtok(input,"\r\n");
  while(lines[linec]!=NULL)
  {
    linec++;
    lines[linec]=strtok(NULL,"\r\n");
  }
  lines[++linec]=NULL;
  linec=0;
  while(lines[linec]!=NULL)
  {
    argv=pmalloc(p,sizeof(char*)*20);
    for(argc=0;argc<=19;argc++) argv[argc]=NULL;
    argc=0;
    last=lines[linec];
    if(last[0]==':')
    {
      from=last+1;
      last=strchr(from,' ');
      if(last==NULL)
      {
        argc=0;
        linec++;
        continue;
      }
      last[0]='\0';
      last++;
      command=last;
      next=strchr(command,' ');
      if(next==NULL)
      {
        argc=0;
        linec++;
        continue;
      }
      next[0]='\0';
      next++;
    }
    else
    {
      from=NULL;
      command=last;
      next=strchr(command,' ');
      if(next==NULL)
      { 
        argc=0;
        linec++;
        continue;
      }
      next[0]='\0';
      next++;
    }
    while(1)
    {
      argv[argc]=next;
      if(argv[argc][0]==':')
      {
        /* good to the last drop */
        argv[argc]++;
        argv[++argc]=NULL;
        break;
      }
      next=strchr(argv[argc++],' ');
      if(next==NULL)
      {
        argv[argc]=NULL;
        break;
      }
      next[0]='\0';
      next++;
      if(argc>14)
      {
        /* IRC only allows for 15 args */
        pth_mutex_release(&s__m);      
        return 1;
      }
    }
    
    foo=0;
    if(j_strcmp(command,"PING")==0)
    {
      spool sp=spool_new(p);
      /* argv[0] -- server which pinged */
      spooler(sp,"PONG :",argv[0],"\n",sp);
      tstream_write(s->stream,spool_print(sp),-1);
    }
    else if(j_strcmp(command,"303")==0)
    {
      irc_handle_ison(s,p,from,argv);
    }
    else if(j_strcmp(command,"JOIN")==0)
      irc_handle_join(s,p,from,argv);
    else if(j_strcmp(command,"ERROR")==0)
      irc_handle_error(s,p,from,argv);
    else if(j_strcmp(command,"PART")==0)
      irc_handle_part(s,p,from,argv);
    else if(j_strcmp(command,"QUIT")==0)
      irc_handle_quit(s,p,from,argv);
    else if(j_strcmp(command,"KICK")==0)
      irc_handle_kick(s,p,from,argv);
    else if(j_strcmp(command,"NICK")==0)
      irc_handle_nick_change(s,p,from,argv);
    else if(j_strcmp(command,"MODE")==0)
      irc_handle_mode(s,p,from,argv);
    /* whois replies */
    else if(j_strcmp(command,"311")==0||j_strcmp(command,"317")==0)
    { /*whis is reply*/
        spool sp=spool_new(p);
        spooler(sp,argv[1]," is ",argv[2],"@",argv[3]," ",argv[4]," ",argv[5],sp);
        irc_send_message(s,NULL,NULL,"groupchat",spool_print(sp),NULL);
    }
    else if(j_strcmp(command,"376")==0)
    {
    	if(s->channel_list!=NULL)
    	{
            spool output=spool_new(p);
            spooler(output,"JOIN ",s->channel_list->channel,"\n",output);
            tstream_write(s->stream,spool_print(output),-1);
        }
    }
    /* TOPIC */
    else if(j_strcmp(command,"332")==0)
    {
      /* argv[0] -- your nick
      argv[1] -- channel
      argv[2] -- the topic
      */
      irc_send_message(s,argv[1],NULL,"groupchat",argv[2],argv[2]);
    }
    /* ROSTER */
    else if(j_strcmp(command,"353")==0)
      irc_handle_roster(s,p,from,argv);
    /* two arg. error messages */
    else if(j_strcmp(command,"256")==0|| /* RPL_ADMINME:         <server> :Administrative info */
            j_strcmp(command,"301")==0|| /* RPL_AWAY:            <nick> :<away message> */
            j_strcmp(command,"313")==0|| /* RPL_WHOISOPERATOR    <nick> :is an IRC operator */
            j_strcmp(command,"315")==0|| /* RPL_ENDOFWHO:        <name> :End of /WHO list */
            j_strcmp(command,"318")==0|| /* RPL_ENDOFWHOIS       <nick> :End of /WHOIS list */
            j_strcmp(command,"319")==0|| /* RPL_WHOISCHANNELS    <nick> :{[@|+]<channel><space>} */
            j_strcmp(command,"331")==0|| /* RPL_NOTOPIC:         <channel> :No topic is set */
            j_strcmp(command,"332")==0|| /* RPL_TOPIC:           <channel> :topic */
            j_strcmp(command,"342")==0|| /* RPL_SUMMONING:       <user> :Summoning user to IRC */
            j_strcmp(command,"353")==0|| /* RPL_NAMERPLY         <channel> :[[@|+]<nick> [[@|+]<nick> [...]]] */
            j_strcmp(command,"369")==0|| /* RPL_ENDOFWHOWAS      <nick> :End of WHOWAS */
            j_strcmp(command,"382")==0|| /* RPL_REHASHING        <config file> :Rehashing */
            j_strcmp(command,"391")==0|| /* RPL_TIME             <server> :<string showing server's local time> */
            j_strcmp(command,"401")==0|| /* ERR_NOSUCHNICK       <nickname> :No such nick/channel */
            j_strcmp(command,"402")==0|| /* ERR_NOSUCHSERVER     <server name> :no such server */
            j_strcmp(command,"403")==0|| /* ERR_NOSUCHCHANNEL    <channel name> :No such channel */
            j_strcmp(command,"404")==0|| /* ERR_CANNOTSENTTOCHAN <channel name> :No such channel */
            j_strcmp(command,"405")==0|| /* ERR_TOOMANYCHANNELS  <channel name> :You have joined too many channels */
            j_strcmp(command,"406")==0|| /* ERR_WASNOSUCHNICK    <nickname> :There was no such nickname */
            j_strcmp(command,"407")==0|| /* ERR_TOOMANYTARGETS   <target> :Duplicate recipients. No message delivered */
            j_strcmp(command,"413")==0|| /* ERR_NOTOPLEVEL       <mask> :No toplevel domain specified */
            j_strcmp(command,"414")==0|| /* ERR_WILDTOPLEVEL     <mask> :Wildcard in TLD */
            j_strcmp(command,"421")==0|| /* ERR_UNKNOWNCOMMAND   <command> :Unknown command */
            j_strcmp(command,"423")==0|| /* ERR_NOADMININFO      <server> :No administrative info available */
            j_strcmp(command,"432")==0|| /* ERR_ERRONEUSNICKNAME <nick> :Erroneus nickname */
            j_strcmp(command,"436")==0|| /* ERR_NICKCOLLISION    <nick> :Nickname collision KILL */
            j_strcmp(command,"442")==0|| /* ERR_NOTONCHANNEL     <channel> :You're not on that channel */
            j_strcmp(command,"444")==0|| /* ERR_NOLOGIN          <user> :user not logged in */
            j_strcmp(command,"461")==0|| /* ERR_NEEDMOREPARAMS   <command> :Not enough parameters */
            j_strcmp(command,"467")==0|| /* ERR_KEYSET           <channel> :Channel key already set */
            j_strcmp(command,"471")==0|| /* ERR_CHANNELISFULL    <channel> :Cannot join channel (+l) */
            j_strcmp(command,"472")==0|| /* ERR_UNKNOWNMODE      <char> :is unknown mode char to me */
            j_strcmp(command,"473")==0|| /* ERR_INVITEONLYCHAN   <channel> :Cannot join channel (+i) */
            j_strcmp(command,"474")==0|| /* ERR_BANNEDFROMCHAN   <channel> :Cannot join channel (+b) */
            j_strcmp(command,"475")==0|| /* ERR_BADCHANNELKEY    <channel> :Cannot join channel (+k) */
            j_strcmp(command,"482")==0)  /* ERR_CHANOPRISNEEDED  <channel> :You're not channel operator */
      irc_handle_error2arg(s,p,from,argv);
    else if(j_strcmp(command,"312")==0|| /* RPL_WHOISSERVER      <nick> <server> :<server info> */ 
            j_strcmp(command,"433")==0|| /* ERR_NICKNAMEINUSE    <nick> :Nickname is already in use */
            j_strcmp(command,"441")==0|| /* ERR_USERNOTINCHANNEL <nick> <channel> :They aren't on that channel */ 
            j_strcmp(command,"443")==0)  /* ERR_USERONCHANNEL    <nick> <channel> :is already on channel */
      irc_handle_error3arg(s,p,from,argv);
    /* server notices */
    else if(j_strcmp(command,"NOTICE")==0)
      irc_handle_notice(s,p,from,argv);
    else if(j_strcmp(command,"PRIVMSG")==0)
      irc_handle_privmsg(s,p,from,argv);
    log_debug(ZONE,"DONE");
    argc=foo=0;
    linec++;
  } 
  LoopCount=0;
  pth_mutex_release(&s__m);      
  return 0;
}                           

