/*
IRC-Transport main.c: Main program loop
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <libxode.h>
#include <irctrans.h>

/*
this is the main function if the listener is to manually connect to etherx via a socket
*/
extern session sessions;
pool main_pool;
pth_mutex_t s__m;
int main(int argc, char *argv[])
/*
returns:
  0 on normal exit
  or other error code to the system
input:
  command line args
*/
{
    pth_event_t sigevt;
    sigset_t set;
    char *config = NULL,*c;
    int help, i, sig;
    main_pool=pool_new();
    help = 0;
    for( i = 1; i < argc; i++ )
    {
        c = argv[i] + 1;
        switch(*c)
        {
        case 'D':
            etherx_debug();
            break;
        case 'c':
            config = pstrdup(main_pool,argv[++i]);
            break;
        default:
            help = 1;
        }
    }
    if( help )
    {
        printf( "Usage: irctransport [-D] [-c config_file] \n");
    }
    else
    {
        /* initialize libjabber and app.* globals */
        log_init(LOGSTDERR, NULL);
        if(load_config(config))
            log_error(ZONE,"[init] Failed to Load config.");
        else
        {
                       
            log_debug(ZONE, "[loading] transport starting up...");

            /* intitialize libetherx and tell it to connect to etherx */
            etherx_init(etherx_secret, etherx_host);
            irc_init();
            sigemptyset(&set);
            sigaddset(&set, SIGHUP);
            sigaddset(&set, SIGINT);
            sigaddset(&set, SIGTERM);
            sigaddset(&set, SIGSEGV);
            pth_sigmask(SIG_UNBLOCK, &set, NULL);
            sigevt = pth_event(PTH_EVENT_SIGS, &set, &sig );
            while(1)
            { /* main loop */
                pth_wait(sigevt);
                if(sig==SIGHUP)
                    load_config(config);
                else break;
            }
        }
    }
    log_error(ZONE,"[exiting] Recieved KILL, shutting down.");
    irc_trans_stop();
    pth_sleep(3);
    pth_kill();
    pool_free(main_pool);
    return 0;
}


