/*
IRC-Transport parser.c: parse commands from jabber clients
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA$
You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <irctrans.h>

/*
handles a message from a jabber user to IRC
*/
int irc_parse_message(jpacket jp)
/*
returns:
 0 on success
 1 if an error occured
input:
 jpacket jp: the incoming jpacket
*/
{
    session s;
    spool sp;
    char *sendmsg;
    if(jp==NULL) return 1;
    pth_mutex_acquire(&s__m,0,NULL);
    s=irc_session_find_by_jid(jp->from,irc_get_server(jp->p,jid_full(jp->to)));
    sp=spool_new(jp->p);
    sendmsg=UTF8_to_str(jp->p,xmlnode_get_data(xmlnode_get_tag(jp->x,"body")));

    if(s==NULL||s->rc==0||sendmsg==NULL) 
    {
        pth_mutex_release(&s__m);        
        jutil_tofrom(jp->x);
        xmlnode_put_attrib(jp->x,"type","error");
        xmlnode_put_attrib(xmlnode_insert_tag(jp->x,"error"),"code","502");
        xmlnode_insert_cdata(xmlnode_get_tag(jp->x,"error"),"Not Connected to IRC server",-1);
        jpacket_reset(jp);
        ehandler_send(ehandler_irc,jp->x,jp->to->server);
        return 1;
    }
    if(strlen(sendmsg)>450)
        sendmsg[450]='\0';
    /*
     * this if block handles the packet.. 
     * each if section is responsible for freeing
     * the packet in it's own way
     */
    if(strncasecmp(sendmsg,"/me ",4)==0)
    {
        /* this should be an action */
        jutil_tofrom(jp->x);
        sendmsg+=4;
        if(jp->to->resource==NULL)
        {
            spooler(sp,jp->to->user,"@",irc_host,"/",s->nick,sp);
            xmlnode_put_attrib(jp->x,"from",spool_print(sp));
            jpacket_reset(jp);
            sp=spool_new(jp->p);
            spooler(sp,"\001ACTION ",sendmsg,"\001",sp);
            svc_irc_line(s,"PRIVMSG",spool_print(sp),1,irc_get_channel(jp->p,jid_full(jp->from)));
            ehandler_send(ehandler_irc,jp->x,jp->to->server);
        }
        else
        {
            /* we'll be nice and strip the @ and + from the user for them */
            char *user=jp->to->resource;
            if(user[0]=='#'||user[0]=='+')user++;
            spooler(sp,"\001ACTION ",sendmsg,"\001",sp);
            svc_irc_line(s,"PRIVMSG",spool_print(sp),1,user);
            tstream_write(s->stream,spool_print(sp),-1);
            xmlnode_free(jp->x);
        }
    }
    else if(strncasecmp(sendmsg,"/say ",5)==0)
    {
        jutil_tofrom(jp->x);
        sendmsg+=5;
        spooler(sp,jp->to->user,"@",irc_host,"/",s->nick,sp);
        xmlnode_put_attrib(jp->x,"from",spool_print(sp));
        xmlnode_hide(xmlnode_get_tag(jp->x,"body"));
        xmlnode_insert_cdata(xmlnode_insert_tag(jp->x,"body"),sendmsg,-1);
        jpacket_reset(jp);
        sp=spool_new(jp->p);
        svc_irc_line(s,"PRIVMSG",sendmsg,1,irc_get_channel(jp->p,jid_full(jp->from)));
        ehandler_send(ehandler_irc,jp->x,jp->to->server);
    }
    else if(strncasecmp(sendmsg,"/raw ",5)==0)
    {
        sendmsg+=5;
        spooler(sp,sendmsg,"\n",sp);
        tstream_write(s->stream,spool_print(sp),-1);    
        xmlnode_free(jp->x);
    }
    else if(strncasecmp(sendmsg,"/quit",5)==0)
    {
        char *reason=sendmsg+5;
        if(reason[0]=='\0') 
            reason=NULL;
        else
            if(reason[0]==' ') reason++;    
        log_debug(ZONE,"LOGGING OFF CHANNEL: %s",reason);
        irc_session_end(s,NULL,reason);
        xmlnode_free(jp->x); 
    }
    else if(strncasecmp(sendmsg,"/part",5)==0)
    {
        char *channel=irc_get_channel(jp->p,jid_full(jp->to));
        char *reason=sendmsg+5;
        if(reason[0]=='\0') 
            reason=NULL;
        else
            if(reason[0]==' ') reason++;    
        log_debug(ZONE,"LOGGING OFF CHANNEL: %s",reason);
        irc_session_end(s,channel,reason);
        xmlnode_free(jp->x); 
    }
    else if(strncasecmp(sendmsg,"/whowas ",7)==0)
    {
        sendmsg+=7;
        svc_irc_line(s,"WHOWAS",NULL,1,sendmsg);
        xmlnode_free(jp->x);
    }
    else if(strncasecmp(sendmsg,"/whois ",7)==0)
    {
        sendmsg+=7;
        svc_irc_line(s,"WHOIS",NULL,1,sendmsg);
        xmlnode_free(jp->x);
    }
    else if(strncasecmp(sendmsg,"/msg ",5)==0||strncasecmp(sendmsg,"/query ",7)==0)
    {
        char *nick;
        if(strncasecmp(sendmsg,"/msg ",5)==0)
            sendmsg+=5;
        else
            sendmsg+=7;
        if(sendmsg!=NULL&&sendmsg[0]!='\0')
        {
            nick=sendmsg;
            sendmsg=strchr(sendmsg,' ');
            if(sendmsg!=NULL)
            {
                sendmsg[0]='\0';
                sendmsg++;
                /* strip the @ and + from the user */
                if(nick[0]=='@'||nick[0]=='+')nick++;
                svc_irc_line(s,"PRIVMSG",sendmsg,1,nick);
                irc_send_message(s,irc_get_channel(jp->p,jid_full(jp->to)),NULL,"groupchat",">> PRIVMSG SENT",NULL);
            }
        }
        xmlnode_free(jp->x);
    }
    else if(strncasecmp(sendmsg,"/mode ",6)==0)
    {
        sendmsg +=6;
        svc_irc_line(s,"MODE",NULL,2,irc_get_channel(jp->p,jid_full(jp->to)),sendmsg);
        xmlnode_free(jp->x);
    }
    else if(sendmsg[0]=='/')
    {
        irc_send_message(s,irc_get_channel(jp->p,jid_full(jp->to)),NULL,"groupchat","Command Not supported",NULL);
        xmlnode_free(jp->x);
    }
    else /* standard message */
    {
        jutil_tofrom(jp->x);
        if(jp->to->resource==NULL)
        {
            spooler(sp,jp->to->user,"@",irc_host,"/",s->nick,sp);
            xmlnode_put_attrib(jp->x,"from",spool_print(sp));
            jpacket_reset(jp);
            sp=spool_new(jp->p);
            svc_irc_line(s,"PRIVMSG",sendmsg,1,irc_get_channel(jp->p,jid_full(jp->from)));
            ehandler_send(ehandler_irc,jp->x,jp->to->server);
        }
        else
        {
            char *user=jp->to->resource;
            if(user[0]=='#'||user[0]=='+') user++;
            svc_irc_line(s,"PRIVMSG",sendmsg,1,jp->to->resource);
            xmlnode_free(jp->x);
        }
    }
    pth_mutex_release(&s__m);
    return 0;
}

/*
handles an unavailable packet from a jabber user
this is used for logging the user out of the channel
*/
int irc_parse_unavailable(jpacket jp)
/*
returns:
 0 if normal, 1 if error
input:
 jpacket jp: the incoming jpacket from etherx
*/
{
    session s;
    if(jp==NULL) return 1;
    
    pth_mutex_acquire(&s__m,0,NULL);
    s=irc_session_find_by_jid(jp->from,irc_get_server(jp->p,jid_full(jp->to)));
    if(s!=NULL&&s->rc!=0)
    { /* the session was found, let's close down that channel */
        log_debug(ZONE,"[parser] channel (%s)[%s] for (%s) going offline", irc_get_channel(jp->p,jid_full(jp->to)),irc_get_server(jp->p,jid_full(jp->to)),jid_full(jp->from));
        irc_session_end(s,irc_get_channel(jp->p,jid_full(jp->to)),NULL);
    }
    pth_mutex_release(&s__m);
    xmlnode_free(jp->x);
    return 0;
}

/*
handles an available packet from jabber client
this is used to log into an IRC channel
*/
int irc_parse_available(jpacket jp)
/*
returns:
 0 on success
 1 if error
input:
 jpacket jp: the incoming jpacket from etherx
*/
{
    session s;
    if(jp==NULL) return 1;
    
    /* check for an existing session */
    pth_mutex_acquire(&s__m,0,NULL);
    s=irc_session_find_by_jid(jp->from,irc_get_server(jp->p,jid_full(jp->to)));
    if(jp->to->resource==NULL||jp->to->resource[0]=='\0')
    {
        pth_mutex_release(&s__m);
        irc_send_error(jp->from,irc_host,"406","Blank resource Not allowed");
        xmlnode_free(jp->x);
        return 1;
    }
    if(s==NULL)
    {
        s=irc_find_default_session(jp->from);
        if(s!=NULL&&s->rc!=0)
            if(strcasecmp(irc_get_server(jp->p,jid_full(jp->to)),s->server)!=0)
                s=NULL;  /* these aren't the servers you're looking for */
    } /* move along */
    if(s!=NULL&&s->rc!=0)
    {
        char *resource=jp->to->resource;
        char *channel=irc_get_channel(jp->p,jid_full(jp->to));
        chans curr=irc_find_channel_by_name(s,channel);
        /* user is already logged in */
        /* check to see if this is a new channel */
        /* Change Nick First.. */
        if(resource!=NULL&&strcasecmp(resource,s->nick)!=0)
        { /* this nick is different than the one we have, do a nick change */
            spool sp=spool_new(jp->p);
            spooler(sp,"NICK ",jp->to->resource,"\n",sp);
            tstream_write(s->stream,spool_print(sp),-1);
        }
        if(curr==NULL)
        { /* this channel wasn't found, join it */
            chans new=NULL;
            pool p=pool_new();
            spool sp=spool_new(jp->p);
            s->rc++;
            new=pmalloc(p,sizeof(_chans));
            new->p=p;
            new->id=pstrdup(p,jp->to->user);
            new->channel=irc_get_channel(p,jid_full(jp->to));
            new->user_list=NULL;
            /* add to begining of list */
            new->next=s->channel_list;
            s->channel_list=new;
            spooler(sp,"JOIN ",channel,"\n",sp);
            tstream_write(s->stream,spool_print(sp),-1);
        }
        pth_mutex_release(&s__m);
        xmlnode_free(jp->x);
        return 0;
    }
    /* start a session here session_create will free the jpacket */
    pth_mutex_release(&s__m);
    irc_session_create(jp,0,NULL);
    return 0;
}

/*
the main parsing function for handling packets from etherx
*/
int irc_parse_packet(jpacket jp)
/*
returns:
 0 on success
 1 on errors
input:
 jpacket jp: the packet from etherx
*/
{
    int retval=0;
    if(jp==NULL) return 1;
    switch( jp->type )
    {
    case JPACKET_IQ:
        switch(jpacket_subtype(jp))
        {
        case JPACKET__GET:
            if(NSCHECK(jp->iq,NS_AGENTS))
            {
                xmlnode query;
                jutil_iqresult(jp->x);
                query=xmlnode_insert_tag(jp->x,"query");
                xmlnode_put_attrib(query,"xmlns","jabber:iq:agents");
                xmlnode_insert_node(query,xmlnode_get_firstchild(irc_agents));
                jpacket_reset(jp);
                ehandler_send(ehandler_irc,jp->x,jp->to->server);
            }
            else if(NSCHECK(jp->iq,NS_VERSION))
            {
                xmlnode iq=jutil_iqresult(jp->x);
                jpacket_reset(jp);
                xmlnode_insert_cdata(xmlnode_insert_tag(iq,"name"),"IRC Transport",-1);
                ehandler_send( ehandler_irc, iq, jp->to->server );
            }
            else if(NSCHECK(jp->iq,NS_TIME))
            {
                xmlnode iq=jutil_iqresult(jp->x);
                jpacket_reset(jp);
                xmlnode_insert_cdata( xmlnode_insert_tag( iq, "utc"),jutil_timestamp(),-1);
                xmlnode_insert_cdata(xmlnode_insert_tag(iq,"tz"),"gmt",-1);
                ehandler_send(ehandler_irc,iq,jp->to->server);
            }
            else
                xmlnode_free(jp->x);
            break;
        case JPACKET__SET:
            if(NSCHECK(jp->iq,NS_OOB))
            {
                xmlnode_free(jp->x);
                /* support for DCC send/gets? */
            }
            else
                xmlnode_free(jp->x);
            break;
        default:
            xmlnode_free(jp->x);
        };
        break;
    case JPACKET_MESSAGE:
        if(jpacket_subtype(jp)==JPACKET__ERROR)
            xmlnode_free(jp->x);
        else
            retval=irc_parse_message(jp);
        break;
    case JPACKET_PRESENCE:
        switch(jpacket_subtype(jp))
        {
        case JPACKET__UNAVAILABLE:
            retval=irc_parse_unavailable(jp);
            break;
        case JPACKET__AVAILABLE:
            retval=irc_parse_available(jp);
            break;
        default:
            xmlnode_free(jp->x);
	}
        break;
    default:
        xmlnode_free(jp->x);
    }
    return retval;
}

